import java.io.*;
import java.util.*;


/**
 * 
 * @author Johannes A. Stork
 *
 */
public class Indexer {
	
	/// Name of the file, that contains a list of all content files
	private String fileIndexFileName;
	
	Indexer(String fileIndexFileName) {
		this.fileIndexFileName = fileIndexFileName;
	}

	/**
	 * Creates a index from the content files
	 * @return Inverted index of the content of all files named in the file index file
	 */
	public void fillIndex(SortedIndex index) {
		/// Prepare a list of all content files
		Vector<String> list = this.getFileList(this.fileIndexFileName);
		
		final long START_TIME = System.currentTimeMillis(); 
		long minRoundTime = Long.MAX_VALUE;
		long maxRoundTime = 0;
		long avgRoundTime = 0;
		int rounds = 0;
		int bk = 0;
		
		/// Look a each content file
		for(String contentFileName : list) {
			long roundStartTime = System.currentTimeMillis(); 
			
			/// Load the content
			StringBuffer content = this.getFileContent(new File(contentFileName));
			
			index.addContent(content, contentFileName);

			rounds++;
			long roundEndTime = System.currentTimeMillis();
			final long roundTime = roundEndTime - roundStartTime;
			if(roundTime > maxRoundTime)
				maxRoundTime = roundTime;
			if(roundTime < minRoundTime)
				minRoundTime = roundEndTime;
			avgRoundTime = (roundEndTime - START_TIME) / rounds;
		
			System.out.println(index);
			System.out.println(rounds + ": cur " + roundTime / 1000.0 + " avg " + avgRoundTime / 1000.0 + " max " + maxRoundTime / 1000.0);
			System.out.println("");
			
			if(rounds % 1000 == 0) {
				index.writeIndex(this.fileIndexFileName + ".idx." + bk );
				bk++;
				bk = bk % 2;
				System.out.println("b");
				System.out.println("");
			}
		}
	}
	
	/**
	 * Reads the file index file
	 * @param indexFileName
	 * @return A list of all files named in the file index file
	 */
	private Vector<String> getFileList(String indexFileName) {
		Vector<String> list = new Vector<String>();
		
		try {
			File indexFile = new File(indexFileName);
			
			BufferedReader br = new BufferedReader(new FileReader(indexFile));
			
			while(br.ready()) {
				File inputFile = new File(indexFile.getParentFile() + "/" + br.readLine());
				if(inputFile.isFile()) {
					list.add(inputFile.toString());
				}
			}
		}
		catch(Exception e) {
			e.printStackTrace(System.err);
		}
		
		return list;
	}
	
	/**
	 * Reads content fo a file
	 * @param inputFile
	 * @return a buffer of the content of the given file
	 */
	private StringBuffer getFileContent(File inputFile) {
		/// Prepare the buffer
		StringBuffer content = new StringBuffer();
		
		try {
			/// Prepare file reading
			BufferedReader br = new BufferedReader(new FileReader(inputFile));
			
			/// Look at each line
			while(br.ready()) {
				content.append(br.readLine() );
				/// The buffered reader removes the white space so put it back
				content.append(" ");
			}
		}
		catch(Exception e) {
			e.printStackTrace(System.err);
		}
		return content;
	}
}

