import java.io.File;
import java.io.PrintWriter;
import java.util.Collections;
import java.util.Iterator;
import java.util.Vector;

/**
 * 
 * @author Johannes A. Stork
 *
 */
public class Runner {

	/**
	 * @param args with name of file list at 0
	 */
	public static void main(String[] args) {
		
		/// Print the argument to the user
		System.out.println("Argument was " + args[0]);
		
		final String FILE_INDEX_FILE_NAME = args[0];
		final String BACKUP_FILE_NAME = args[0] + ".idx";
		final String STAT_FILE_NAME = args[0] + ".dat";

		/// Prapare empty index 
		SortedIndex sortedIndex = new SortedIndex();
		
		if(false) {
			System.out.println("Create index");
			/// Prepare new Indexer instance
			Indexer indexer = new Indexer(FILE_INDEX_FILE_NAME);
			
			/// Let the indexer create a new index from parsing the content files
			indexer.fillIndex(sortedIndex);	
		} else {
			System.out.println("Load index");
			sortedIndex.loadIndex(BACKUP_FILE_NAME);
		}
		
		System.out.println("Index ready: " + sortedIndex);
		
		/// Let the indexe write the index to a file
		sortedIndex.writeIndex(BACKUP_FILE_NAME);
		
		searchTwoWordOneHit(sortedIndex);
		double epsilon = getZipfsEpsilonFromLeastSquareError(sortedIndex);
		System.out.println("Epsilon " + epsilon);
		
		//searchTwoWordOneHit(sortedIndex);
		
		/// Sort the index by frequency of the words
		//Vector<IndexElement> index = sortedIndex.getIndex();
		//Collections.sort(index, new IndexElementFrequencyComparator());
		//writeStatistics(STAT_FILE_NAME, index);
	}
	
	/**
	 * Searches the given index for all two word queries with only one hit
	 * @param sortedIndex
	 */
	public static void searchTwoWordOneHit(SortedIndex sortedIndex) {
		Vector<IndexElement> index = sortedIndex.getIndex();
		
		for(int i = 0; i < index.size(); i++) {
			for(int j = i + 1; j < index.size(); j++) {
				IndexElement i1 = index.get(i);
				IndexElement i2 = index.get(j);
				
				if(!i1.content.equals(i2.content)) {
					Vector<String> hits = doItersection(i1.fileNames, i2.fileNames);
					if(hits.size() == 1) {
						System.out.println(i1.content + " -- " + i2.content);
					}
				}
			}
		}

	}
	
	public static Vector<String> doItersection(Vector<String> l1, Vector<String> l2) {
		/// Prepare return value
		Vector<String> intersection = new Vector<String>();
			
		/// Calculate intersection of the hit lists
		Iterator<String> it1 = l1.iterator();
		Iterator<String> it2 = l2.iterator();
			
		/// Initial values
		String s1 = "";
		String s2 = "";
			
		/// Compare metric value
		int c = s1.compareTo(s2);
		
		/**
		 * c == 0: s1 eq s2
		 * c < 0: s1 lex< s2
		 * c > 0: s1 lex> s2
		 * 
		 *  @see Java API
		 */
		
		while((c == 0 && it1.hasNext() && it2.hasNext()) ||
				(c < 0 && it1.hasNext()) ||
				(c > 0 && it2.hasNext())) {
			
			if(c == 0) {
				/// Hits are equal
				
				/// Move the iterators
				s1 = it1.next();
				s2 = it2.next();
			} else if(c < 0) {
				/// s1 < s2
		
				s1 = it1.next();
			} else {
				/// s1 > s2
				
				s2 = it2.next();
			}
			
			c = s1.compareTo(s2);
			
			if(c == 0) {
				/// Intersection found
				
				intersection.add(s1);
			}
		}
		
		return intersection;
	}
	
	/**
	 * Write values for the plot
	 * @param fileName
	 * @param index
	 */
	public static void writeStatistics(String fileName, Vector<IndexElement> index) {
		try{
			File file = new File(fileName);
			PrintWriter pw = new PrintWriter(file);
			

			double n = index.size();
			int j = 0;
			
			for(IndexElement i : index) {
				double pos = j + 1;
				double occ = i.occurences;

				//double epsilon = occ / (n / (1.0/pos));
				
				
				pw.println(pos + " " + occ);
				j++;	
			}
			
			pw.flush();
			pw.close();
		} 
		catch(Exception e) {
			e.printStackTrace(System.err);
		}
	}
	
	/**
	 * Least square error fitting for epsilon from Zipf
	 * @param sortedIndex
	 * @return
	 */
	public static double getZipfsEpsilonFromLeastSquareError(SortedIndex sortedIndex) {
		/// The number of overall occorances
		double n = sortedIndex.getOccurences();
		
		/// Prepare list of measured values
		Vector<Double> values = new Vector<Double>();
		
		/// Interal data structure of the sorted index
		Vector<IndexElement> index = sortedIndex.getIndex();
		
		/// Apply sorting by frequancy
		Collections.sort(index, new IndexElementFrequencyComparator());
		
		//double lastValue = Double.MAX_VALUE;
		
		double j = 0.0;
		for(IndexElement ie : index) {
			j+= 1.0;
			
			double value = (double) ie.occurences;
			values.add(value);
			
			//System.out.println(value * j / (n));
		}
		
		
		
		
		double step = 0.01;
		
		double epsilon = 0.0;
		
		double lastError = Double.MAX_VALUE;
		
		for(int i = 0; i < 5000; i++) {
			
			epsilon = epsilon + step;
			
			double currentError = getSquareError(values, n, epsilon);
			
			if(currentError < lastError) {
				lastError = currentError;
			} else {
				step = -0.9 * step;
			}
			
		}
		
		return epsilon;
	}
	
	/**
	 * Error metric
	 * @param values
	 * @param n The number of overall occurences
	 * @param epsilon
	 * @return the square error
	 */
	private static double getSquareError(Vector<Double> values, double n, double epsilon)  {
		double error = 0.0;
		
		for(int i = 0; i < values.size(); i++) {
			/// Value according to formular
			double v1 = epsilon * n * (1.0 / (i +1));
			
			/// Square error
			error += (v1 - values.get(i)) * (v1 - values.get(i));
		}
		
		return error;
	}
}
